//------------------------------------------------------------------------------
//
//  spi.c
// 	
//	Initialization of the CED80 and peripherials
//
//
// 	Juergen Austen,  Texas Instruments, Inc
//  December 2008
//
//	Revision history:
//	0.1 	27-Nov		First implementation
//
//
//------------------------------------------------------------------------------

#include  <msp430x12x.h>
#include  <defines.h>
#include  <Board defines.h>

/******************************************************************************/

/******************************************************************************/
// Basic function from SPI_LowLevel
unsigned char MSP430_SPI_Tx(unsigned char *TxBuffer, unsigned char size);


/******************************************************************************/
extern unsigned char SW_Data,ucTX_ONLY;
#define MAX_BUF_SIZE      16

/******************************************************************************/
struct stSPI_Stack
{
  unsigned char ucOutput[MAX_BUF_SIZE];
  unsigned char ucInput[MAX_BUF_SIZE];
  unsigned char ucOutputPointer;
};

/******************************************************************************/
struct _st_TRP_CONTENT  TRP_Data;
struct stSPI_Stack      SPI_Stack;



unsigned char ucTX_ONLY;
extern unsigned char ucWDT_Count;

/******************************************************************************/
void MSP430_SPI_Init(void)
 /****************************************************************************************
  *
  *  Initialization of the SPI Interface
  *  If STANDARD_SPI is defined the SPI interface of the MSP430F123 is used
  *  otherwise an emulation of the SPI is used using the pins:
  *
  *   SPI_SIMO	  // P2.1 Output
  *   SPI_SOMI	  // P2.2 Input
  *   SPI_CLK 	  // P2.3 Output
  *   TMS_BUSY	  // P2.4 Input
  *
  ****************************************************************************************/
{
  P3OUT = 0;
  P3SEL = SPI_SIMO + SPI_SOMI + SPI_CLK;
	U0CTL = CHAR + SYNC + MM + SWRST;     						// 8-bit, SPI, Master
	U0TCTL = SSEL1 + STC;          							// SMCLK, 3-wire
	U0BR0 = 0x03;                        						// SPICLK = SMCLK/3(About 660KHz)
	U0BR1 = 0x00;
	U0MCTL = 0x00;
	ME2 = USPIE0;
  U0CTL &= ~SWRST;
}


//******************************************************************************

void ErrorMode(void)
{
	WDTCTL = WDT_MRST_0_064;
	//for(;;);					// MCU restart....
}
/******************************************************************************/

void Start_SPI_WDT(void)
{
    WDT_HOLD();
/*
    ucWDT_Count = 0;
    WDT_INTERVAL_MODE();                      // Start timer in interval mode
    IE1 |= WDTIE;
    _EINT();                                  // Enable interrupts
    */ //Error
}

void Stop_SPI_WDT(void)
{
    WDT_HOLD();
    ucWDT_Count = 0;
    IE1 &= ~WDTIE;
}


/******************************************************************************/
unsigned char MSP430_SPI_Rx(unsigned char *RxBuffer, unsigned char size)
 /****************************************************************************************
  *
  *  Receive a telegram with the SPI Emulation
  *
  ****************************************************************************************/
{
    if(!size) return FALSE;					        		// invalid data size
    Start_SPI_WDT();

    size--;
	  do
	  {
	    TXBUF0 = 0x00;
	    while (!(U0TCTL & TXEPT));
      while((P2IN & P2_BUSY) == P2_BUSY);
	    while(!(P2IN & P2_BUSY));
   	  *RxBuffer++ = RXBUF0;
	  }while(--size);
	
    TXBUF0 = 0x00;
    while (!(U0TCTL & TXEPT));
    while((P2IN & P2_BUSY) == P2_BUSY);
    *RxBuffer = RXBUF0;

    return 0x00;
}



/******************************************************************************/
unsigned char MSP430_SPI_Tx(unsigned char *TxBuffer, unsigned char size)
{
    if(!size) return FALSE;					        		// invalid data size
    Start_SPI_WDT();


    if((P2IN & P2_BUSY) == P2_BUSY) return FALSE;	  		        	// Control Unit is not ready(Busy is high)
    if (!(U0TCTL & TXEPT)) return FALSE;						// MSP430 SPI Tx buffer is not ready.
	  do
    	 {
            TXBUF0 = *TxBuffer++;
            while (!(U0TCTL & TXEPT));
            while((P2IN & P2_BUSY) == P2_BUSY);
            if (ucTX_ONLY && size == 0x01)
                ;                           // Only transmit data to frontend then busy line stays low after last SPI byte !
            else
                while(!(P2IN & P2_BUSY));                                               // wait until busy goes from low to high again
	      }while(--size);
    Stop_SPI_WDT();
	  return TRUE;
}



void SPI_Buf_Reset(void)
{
 /****************************************************************************************
  *
  *  Set Buffer Pointer to second position
  *  First position is reserved for the length byte
  *  and could be accessed by the function SPI_Buf_Set_Telegram_Length()
  *
  ****************************************************************************************/

 SPI_Stack.ucOutputPointer = 1;
}

void SPI_Buf_Set_Telegram_Length(void)
{
 /****************************************************************************************
  *
  *  Directly access the output buffer on the position 0 --> Length of the telegram
  *
  ****************************************************************************************/
  if (SPI_Stack.ucOutputPointer)
    SPI_Stack.ucOutput[0] = SPI_Stack.ucOutputPointer-1;
}

void SPI_Buf_Set_Output_Byte(unsigned char ucByte)
{
 /****************************************************************************************
  *
  *  Directly access the output buffer on the position 0 --> Length of the telegram
  *
  ****************************************************************************************/
  if (SPI_Stack.ucOutputPointer >= MAX_BUF_SIZE)
    {
      ErrorMode();
    }
  else
    {
    SPI_Stack.ucOutput[SPI_Stack.ucOutputPointer] = ucByte;
    SPI_Stack.ucOutputPointer++;
    }
}

void SPI_Buf_Send(void)
{
 /****************************************************************************************
  *
  *  Send the Buffer to the Frontend
  *
  ****************************************************************************************/
   if (MSP430_SPI_Tx(SPI_Stack.ucOutput,SPI_Stack.ucOutput[0]+1))
      ErrorMode();
}

void SPI_Set_Up_Telegram(void)
{ /****************************************************************************************
  *
  *  This set up is equal for each telegram
  *
  ****************************************************************************************/
    SPI_Buf_Reset();                                                            // Reset the Send Buffer Pointer
    SPI_Buf_Set_Output_Byte(TAC_COMMAND_BYTE);                                  // Do a TAC command
}

/**********************************  Transponder Access Commands (TAC) **********************************/

/******************************************************************************/
void SPI_Read_SerialNum(void)
 /****************************************************************************************
  *
  *  Read out serial number, User data 1 and Password
  *
  ****************************************************************************************/
{
  SPI_Set_Up_Telegram();                                 
  SPI_Buf_Set_Output_Byte(Page3);                        
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();

  if (MSP430_SPI_Rx(SPI_Stack.ucInput,7))
    ErrorMode();

  TRP_Data.SelectiveAddress = SPI_Stack.ucInput[0];
  TRP_Data.KeyNumber        = SPI_Stack.ucInput[1];      // equal to User data 1
  TRP_Data.SerialNumber[0]  = SPI_Stack.ucInput[2];
  TRP_Data.SerialNumber[1]  = SPI_Stack.ucInput[3];
  TRP_Data.SerialNumber[2]  = SPI_Stack.ucInput[4];
  TRP_Data.SerialNumber[3]  = SPI_Stack.ucInput[5];
}


/******************************************************************************/
void SPI_Read_UserPage(unsigned char ucPage, unsigned char *ucData)
{unsigned char i;

  SPI_Set_Up_Telegram();                                  
  SPI_Buf_Set_Output_Byte(ucPage);                                              // Read page
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();

  if(MSP430_SPI_Rx(SPI_Stack.ucInput,7))
    ErrorMode();

  for (i=0; i<5; i++)
  {
    ucData[i] = SPI_Stack.ucInput[i+1];
  }
}

/******************************************************************************/
void SPI_Program_UserPage(unsigned char ucPage, unsigned char *ucData)
{
  unsigned char i;

  SPI_Set_Up_Telegram();                                 // Enter CSP and if needed password
  SPI_Buf_Set_Output_Byte(ucPage+1);                     // Select command 'Program Page X'
  if (TRP_Data.SelectiveAddress != 0xFF)
    {// Selective address required for programming
      SPI_Buf_Set_Output_Byte(TRP_Data.SelectiveAddress);
    }
  for (i=0; i<5; i++)
    {
      SPI_Buf_Set_Output_Byte(ucData[i]);
    }
  SPI_Buf_Set_Telegram_Length();
  SPI_Buf_Send();

  if(MSP430_SPI_Rx(SPI_Stack.ucInput,7))                  // Get response
    ErrorMode();
  
  for (i=0; i<5; i++)
    {
      ucData[i] = SPI_Stack.ucInput[i+1];
    }
}

/******************************************************************************/
void SPI_Program_PushButtonMask(void)
{
  SPI_Set_Up_Telegram();                                  
  SPI_Buf_Set_Output_Byte(Page18+1);                      
  SPI_Buf_Set_Output_Byte(TRP_Data.PushButtonMask);
  SPI_Buf_Set_Telegram_Length();
  SPI_Buf_Send();

  if(MSP430_SPI_Rx(SPI_Stack.ucInput,7))                  // Get response
    ErrorMode();

  TRP_Data.PushButtonMask = SPI_Stack.ucInput[0];
}


/******************************************************************************/
void SPI_Read_PushButtonMask(void)
{
  SPI_Set_Up_Telegram();                                 // Enter CSP and if needed password
  SPI_Buf_Set_Output_Byte(Page18);                       // Select command 'Read Page X'
  SPI_Buf_Set_Telegram_Length();
  SPI_Buf_Send();

  if(MSP430_SPI_Rx(SPI_Stack.ucInput,7))                  // Get response
    ErrorMode();

  TRP_Data.PushButtonMask = SPI_Stack.ucInput[0];
}


/***************************************  Enhanced Commands (EC) ****************************************/

void SPI_Read_PCU_State(void)
{
 /****************************************************************************************
  *
  *  Programm Rolling code register and receive RC and TIC
  *
  ****************************************************************************************/

  SPI_Buf_Reset();                                       // Reset the Send Buffer Pointer
  SPI_Buf_Set_Output_Byte(MSP430Read_CU_Status);         // Command: Read Status from PCU
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();

  // Read out response
  if(MSP430_SPI_Rx(SPI_Stack.ucInput,2))//Error should be 2
    ErrorMode();
  switch (SPI_Stack.ucInput[0])
  {
    case 1:
      TRP_Data.ucPCU_Mode = PCU_PUSH_BUTTON;
      break;
    case 2:
      TRP_Data.ucPCU_Mode = PCU_MSP_ACCESS;
      break;
    default:
      ErrorMode();
      break;
  }
}

void SPI_Power_Down(void)
{
 /****************************************************************************************
  *
  *  Send Power down command: After this command
  *
  ****************************************************************************************/
  ucTX_ONLY = TRUE;
  SPI_Buf_Reset();                                       // Reset the Send Buffer Pointer
  SPI_Buf_Set_Output_Byte(MSP430_PowerDownMode);         // Command: Read Status from PCU
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();
  ucTX_ONLY = FALSE;
}

void SPI_Encrypt_Data(unsigned char *ucData, unsigned char *ucSig)
{ unsigned char i;

  SPI_Buf_Reset();                                       // Reset the Send Buffer Pointer
  SPI_Buf_Set_Output_Byte(CU_Encryption);                // Command: Encryption
  for (i=0; i<5; i++)
    {
      SPI_Buf_Set_Output_Byte(ucData[i]);
    }
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();

  // Read out response
  if(MSP430_SPI_Rx(SPI_Stack.ucInput,3))
    ErrorMode();
  ucSig[0] = SPI_Stack.ucInput[0];
  ucSig[1] = SPI_Stack.ucInput[1];
  ucSig[2] = SPI_Stack.ucInput[2];
}

void SPI_CRC_Calc(unsigned char ucStart,unsigned char ucLength, unsigned char *ucData, unsigned char *ucCRC)
{ unsigned char i;

  SPI_Buf_Reset();                                       // Reset the Send Buffer Pointer
  SPI_Buf_Set_Output_Byte(CU_CRC_Calculation);           // Command: CRC calculation with Start value 0x3791
  SPI_Buf_Set_Output_Byte(ucLength-ucStart);             // Send Length of data stream
  for (i=ucStart; i<ucLength; i++)
    {
      SPI_Buf_Set_Output_Byte(ucData[i]);
    }
  SPI_Buf_Set_Telegram_Length();

  SPI_Buf_Send();

  // Read out response
  if(MSP430_SPI_Rx(SPI_Stack.ucInput,2))
    ErrorMode();
  ucCRC[0] = SPI_Stack.ucInput[0];
  ucCRC[1] = SPI_Stack.ucInput[1];

}


