//------------------------------------------------------------------------------
//
//  usb.c
// 	
// 	Interface between the host PC and the BaseStation, via USB.
//
//
// 	Juergen Austen,  Texas Instruments, Inc
//  December 2008
//
//	Revision history:
//	0.1 	27-Nov		First implementation
//
//
//------------------------------------------------------------------------------

#include  <msp430x12x.h>
#include  <defines.h>
#include  <Board defines.h>

// variables
struct				stASCIIbuffer	USB_Buffer;	
struct 				stLFpacket		aLFpacket;
union  				unCMD  			command1;
union  				unCMD2 			command2;

extern unsigned char				ucMCW1, ucMCW2;
extern unsigned int 				uiStopCondition;
extern char rxdata[14];
extern struct stBLC 				BLCtimes;
extern struct stPWM					PWMtimes;
extern struct stTimerCtrl       	TimerCtrl;



void GetPowerBurst(char number, char* Index);


void AppendLRC(char startindex, char stopindex)
{
	// Calculates the LRC from startindex to stopindex and apppends it at stopindex+1.
	char i,lrc=0x00;
	for (i=startindex;i<=stopindex;i++)
	{
		lrc ^= USB_Buffer.ucData[i];
	}
	USB_Buffer.ucData[i] = lrc;
}
//-----------------------------------------------------------------------------------------------

int toInt(char c)
{	
  // 	Interprets (decodes) c as a single-digit hexadecimal value, coded in ASCII.
  //		ASCII characters '0'-'9' generate values 0-9.
  //		ASCII characters 'A'-'F' and 'a'-'f' generate values 10-15. I
  //		If the value of c is not a hexadecimal digit, the function returns 1.
  
	
	if ((c >= 0x30) && (c <= 0x39))			// '0' to '9'
		return (c-0x30);
	else
		if ((c >= 0x41) && (c <= 0x46))		// 'A' to 'F'
			return (c-0x37);
		else
			if ((c >= 0x61) && (c <= 0x66))	// 'a' to 'f'
				return (c-0x57);
			else
				return (-1);
}
//-----------------------------------------------------------------------------------------------

char toASCII(int i)
{
	// 	Codes i to a single-digit hexadecimal value in ASCII.
	//		Values 0-9 generate ASCII characters '0'-'9'.
	//		Values 10-15 generate ASCII characters 'A'-'F'.   I
	//		If the value of i is not a hexadecimal digit, the function returns '?'.
  
	if ((i >= 0) && (i <= 9))
		return (i+0x30);									// '0' to '9'
	else
		if ((i >= 10) && (i <= 15))
			return (i+0x37);								// 'A' to 'F'
		else
			return (0x3F);	
}
//-----------------------------------------------------------------------------------------------

void ReceiveByteUSB(void)
{
	//	This function is uniquely called up from the P1 ISR
	//  It receives one byte on the RX_UART line by using the TimerA
	//	(attention interrupt nesting, TimerA interrupt occurs while P1 interrupt
	//	is still ongoing	
  unsigned int  shift;

  shift = 0x01;
  TimerCtrl.bWaitForTimeout = FALSE;
  
  USB_Buffer.ucData[USB_Buffer.ucLength] = 0;				     	// Reset
  TimerAWait_CCR1(142, HALT);						              	// Wait ~140us or 1.5 bit times at 9600baud to get the Bit0
    
  // ******************* Read byte ********************************
  do
  {  
    if ((P1IN & P1_RX_UART) && (USB_Buffer.ucLength <= 2*RX_BUF_SIZE)) 	// High bit?	Array index within bounds?
       USB_Buffer.ucData[USB_Buffer.ucLength] |= shift; 				// Set current bit in current byte
    shift <<= 1;														// shift left, to next higher bit (bit1, bit2 etc.)
    TimerAWait_CCR1(94, HALT);											// Wait 104us to the next bit
  }
  while (shift <= 0x80);												// until Bit7 is reached
  // ******************** End of byte *****************************
  USB_Buffer.ucLength++;
  TimerCtrl.bWaitForTimeout = TRUE;									    // 
  TimerAWait_CCR2(10000, CONTINUE);			    			            // Activate 10ms Timeout
}	
//-----------------------------------------------------------------------------------------------

void SendByteUSB(unsigned char byte)
{
	// 	Intended for transmitting an ASCII coded character to the host PC.
	//	Transmits a byte by shifting from LSB to MSB.

	
	unsigned int shift;
	unsigned char volatile *ucLF_Port;
	unsigned char ucLF_Port_Bit;
	
	ucLF_Port     = &P1OUT;
    ucLF_Port_Bit = P1_TX_UART;
	
	SetOutput(ucLF_Port, ucLF_Port_Bit, HIGH,94, USEC, HALT);	  			// necessary for the PC to check when Startbit begins
    SetOutput(ucLF_Port, ucLF_Port_Bit, LOW ,94, USEC, HALT);				// Start bit (low) ; corrected
	for (shift = 0x01; shift <= 0x80; shift <<= 1)
	{
		if (byte & shift) // High bit
			SetOutput (ucLF_Port, ucLF_Port_Bit, HIGH, 94, USEC, HALT); 	// 104 USEC == 1 bit time @ 9600 baud
		else // Low bit
			SetOutput (ucLF_Port, ucLF_Port_Bit, LOW, 94, USEC, HALT);
	}
	SetOutput(ucLF_Port, ucLF_Port_Bit, HIGH, 195, USEC, HALT);				// Stop bit(high) ; longer time(default=104uis)
        		                                                  			// otherwise PC doesn`t check start of transmission correct
}


void NormalModeUSB(void)
{
	//	Packet interpretation for a normal downlink to the transponder.
	//	2nd Gen. Automotive Reader Packet structure.

	
	char cIndex = 2;			// zero-based index to current data field in USB_Buffer - we start with CMD byte.
	char c;																
	
	// ******************** Write the command byte(s) into the structures ******************************
	command1.usValue = USB_Buffer.ucData[cIndex++];					// First command byte is always present
	if (command1.field.ByteFollows == 1)			
		command2.usValue = USB_Buffer.ucData[cIndex++];				// Second command byte is optional
	else
		command2.usValue = 0;
	
	// ******************* Determine the mode from CMD1 byte (bits 5, 4, 3) **************************
	switch (command1.field.Select1)
		{
		case 0:		
              if (command1.field.Select0)
                  {
                    command1.field.ucMode = PE_WP;			
                  }
                else
                  {
                  command1.field.ucMode = IMMO;			
                  }
      		  break;
		case 1:
            if (!command1.field.Select0)
				{
 				 if (command1.field.x26_x22 == x26)
                 	{
          		      command1.field.ucMode = PE_noWP_26;
                    }
                 else
                    {
	   			     command1.field.ucMode = PE_noWP_22;
                    }
                }
            else
                {
      		     command1.field.ucMode = BBUP;			
                }
		}
	GetPowerBurst(1, &cIndex);										// Extract Power Burst 1
	cIndex++;
		
	// ******************** Additional fields for special packet types ******************************
		
	// Read extra data bytes in Battery Backup or Wake Pattern packets
	if ((command1.field.ucMode == BBUP)	|| (command1.field.ucMode == PE_WP))							
	{
		aLFpacket.ucEXTRAbytes = USB_Buffer.ucData[cIndex++] / 8;		// how many EXTRA bytes (convert from bits)
		// TO DO - CHECK AGAINST MAX SIZE
		for (c=0; c<aLFpacket.ucEXTRAbytes; c++)												
			aLFpacket.ucEXTRAdata[c] = USB_Buffer.ucData[cIndex++];		// get EXTRA data (either BBUP or WP)
		if (command1.field.ucMode == BBUP)
			GetPowerBurst(4, &cIndex);																	// for BBUP get Power Burst 4
	};
	
	// Packets that contain a Power Burst 3
	if ((command1.field.ucMode == BBUP) || (command1.field.ucMode == PE_WP) || (command1.field.ucMode == PE_noWP_26))
	{
		GetPowerBurst(3, &cIndex);																	// get Power Burst 3	
		cIndex++;
	};
	
	// ******************** All packet types : read main fields **************************************
	aLFpacket.ucTXbytes = USB_Buffer.ucData[cIndex++] / 8;					// how many TX bytes (convert from bits)
	for (c=0; c<aLFpacket.ucTXbytes; c++)												
		aLFpacket.ucTXdata[c] = USB_Buffer.ucData[cIndex++];					// get TX data
	GetPowerBurst(2, &cIndex);																			// get Power Burst 2
	cIndex++;
	USB_Buffer.ucRXbytes = USB_Buffer.ucData[cIndex];								// read no. of RX bytes demanded
}


void SetupModeUSB(void)
{
	//  Packet interpretation for microcontroller setup - no downlink to transponder.
	//  Read or write parameters from/to the MCU. USB_Buffer is constructed for the response.
	//		While a downlink packet starts with fields 	Start, Length, Command, ...,
	//		the uplink packet starts with fields				Start, Command, '00', ...
	//		The AutomotiveDemoTool usually sends a series of 4 packets
	//		(MCW, PWM, PPM, Stop condition write), each of which has to be acknowledged.
	
	USB_Buffer.ucData[0]  = 0x01;			// Start
	switch (USB_Buffer.ucData[2])			// command byte
	{
		// ******************* Write / Read PWM times *************************************
	
		case 0x03:				// Write PWM times to MCU (two chars into one int)
                    PWMtimes.usToffH  = USB_Buffer.ucData[3] * 256 + USB_Buffer.ucData[4];
                    PWMtimes.usTonH   = USB_Buffer.ucData[5] * 256 + USB_Buffer.ucData[6];
                    PWMtimes.usToffL  = USB_Buffer.ucData[7] * 256 + USB_Buffer.ucData[8];
                    PWMtimes.usTonL   = USB_Buffer.ucData[9] * 256 + USB_Buffer.ucData[10];
                    // acknowledge is formed from the following code							
		case 0x01:				// Read PWM times from MCU (one int into two chars)
                    USB_Buffer.ucData[1]  = 0x09;			// Command? or length?
                    USB_Buffer.ucData[2]  = 0x00;			// 00
                    USB_Buffer.ucData[3]  = PWMtimes.usToffH / 256;
                    USB_Buffer.ucData[4]  = PWMtimes.usToffH % 256;
                    USB_Buffer.ucData[5]  = PWMtimes.usTonH / 256;
                    USB_Buffer.ucData[6]  = PWMtimes.usTonH % 256;
                    USB_Buffer.ucData[7]  = PWMtimes.usToffL / 256;
                    USB_Buffer.ucData[8]  = PWMtimes.usToffL % 256;
                    USB_Buffer.ucData[9]  = PWMtimes.usTonL / 256;
                    USB_Buffer.ucData[10] = PWMtimes.usTonL % 256;
                    AppendLRC(1,10);								// calculate LRC of fields 1 to 11
                    USB_Buffer.ucLengthActual = 12;
                    break;
		// ******************* Write / Read BLC times *************************************							
		case 0x13:				// Write BLC times to MCU (two chars into one int)
                    BLCtimes.usToff   = 	USB_Buffer.ucData[3] * 256 + USB_Buffer.ucData[4];
                    BLCtimes.usTonH   = 	USB_Buffer.ucData[5] * 256 + USB_Buffer.ucData[6];
                    BLCtimes.usTonSC  = 	USB_Buffer.ucData[7] * 256 + USB_Buffer.ucData[8];
                    BLCtimes.usTonL   = 	USB_Buffer.ucData[9] * 256 + USB_Buffer.ucData[10];
                    // acknowledge is formed from the following code                            
		case 0x11:				// Read BLC times from MCU (one into into two chars)
                    USB_Buffer.ucData[1]  = 0x09;			// command? or length?
                    USB_Buffer.ucData[2]  = 0x00;			// 00
                    
                    USB_Buffer.ucData[3]  = BLCtimes.usToff / 256;
                    USB_Buffer.ucData[4]  = BLCtimes.usToff % 256;
                    USB_Buffer.ucData[5]  = BLCtimes.usTonH / 256;
                    USB_Buffer.ucData[6]  = BLCtimes.usTonH % 256;
                    USB_Buffer.ucData[7]  = BLCtimes.usTonSC / 256;
                    USB_Buffer.ucData[8]  = BLCtimes.usTonSC % 256;
                    USB_Buffer.ucData[9]  = BLCtimes.usTonL / 256;
                    USB_Buffer.ucData[10] = BLCtimes.usTonL % 256;
                    AppendLRC(1,10);							// calculate LRC of fields 1 to 11
                    USB_Buffer.ucLengthActual = 12;
                    break;																	
		// ********************************************************************************					
		case 0x15:				// Write Stop condition
                    uiStopCondition = USB_Buffer.ucData[3] * 256 + USB_Buffer.ucData[4];
                    // acknowledge
                    USB_Buffer.ucData[1]  = 0x15;			// command
                    USB_Buffer.ucData[2]  = 0x00;			// 00
                    USB_Buffer.ucData[3]  = uiStopCondition / 256;
                    USB_Buffer.ucData[4]  = uiStopCondition % 256;
                    AppendLRC(1,4);
                    USB_Buffer.ucLengthActual = 6;
                    break;	
		case 0x05:				// Read software and hardware versions from MCU
                    USB_Buffer.ucData[1]  = 0x05;			// Length from next byte
                    USB_Buffer.ucData[2]  = 0x00;			// 00
                    USB_Buffer.ucData[3]  = GENERAL_SW_VERSION;
                    USB_Buffer.ucData[4]  = UNDER_SW_VERSION;
                    USB_Buffer.ucData[5]  = HW_TYPE;
                    USB_Buffer.ucData[6]  = HW_VERSION;
                    AppendLRC(1,6);								// calculate LRC of fields 1 to 11
                    USB_Buffer.ucLengthActual = 8;
                    break;
		case 0x07:				// Set Mode Control Byte(s) (MCW)
                    ucMCW1 = USB_Buffer.ucData[3];
                    ucMCW2 = USB_Buffer.ucData[4];
                    // acknowledge (w/o length field)
                    USB_Buffer.ucData[1]  = 0x07;
                    USB_Buffer.ucData[2]  = 0x00;			// 00
                    USB_Buffer.ucData[3]  = ucMCW1;
                    USB_Buffer.ucData[4]  = ucMCW2;
                    ucMCW1 |= BIT5;		// always set BIT5 for synchronous data out
                    ucMCW2 |= BIT5;		
                    AppendLRC(1,4);
                    USB_Buffer.ucLengthActual = 6;
                    break;

		// ********************************************************************************
		case 0x09:				// Manual TXCT low
                    P2OUT &= ~BIT0;
                    // acknowledge is formed from the following code
            
		case 0x0B:				// Manual TXCT high
                    if (USB_Buffer.ucData[1] == 0x0B)
                            P2OUT |= BIT0;
                    USB_Buffer.ucData[1]  = 0x01;			
                    USB_Buffer.ucData[2]  = 0x00;			
                    USB_Buffer.ucData[3]  = 0x01;
                    USB_Buffer.ucLengthActual = 4;
                    break;
		// ********************************************************************************									
		case 0x0D:				// antenna selection
                    USB_Buffer.ucData[1]  = 0x01;			
                    USB_Buffer.ucData[2]  = 0x02;			
                    USB_Buffer.ucData[3]  = 0x0D;
                    USB_Buffer.ucLengthActual = 4;
                    break;
		// ********************************************************************************									
	}
}



void GetPowerBurst(char number, char* Index)
{
	// Mini-routine to read Power Burst[number] from the RX packet and increment the Index correspondingly
	// ATTENTION, number is one-based for better readability
	if (command1.field.TwoPB_OnePB == ONE_BYTE)
			aLFpacket.usPowerBurst[number-1] = USB_Buffer.ucData[*Index];																		// One-byte Power Burst
	else
			aLFpacket.usPowerBurst[number-1] = 16 * USB_Buffer.ucData[*Index] + USB_Buffer.ucData[(*Index)+1];	// Two-byte Power Burst
	*Index++;
}


int DownlinkUSB(void)
{
	//  Decodes the ASCII packet buffer. Checks the Start, Length and LRC bytes.
	//  Does a quick check of the CMD byte to extract the mode.
	//	Returns  0 for a good packet, normal mode.
	//	Returns  1 for a good packet, setup mode.
	//	Returns  2 for a good packet, probe test mode.
	//	Returns -1 for a bad packet, LRC error.
	//	Returns -2 for a bad packet, Start or Length byte error.
	unsigned char i,ucLen,lrc_calc=0x00;
	signed int m,n;
	
	ucLen = 0;
	for (i=0 ; i<USB_Buffer.ucLength/2 ; i++)
	{
		// Combine the two ASCII characters into one byte and add to packet array
		m = toInt(USB_Buffer.ucData[2*i]) * 16;						    // high byte		
		n = toInt(USB_Buffer.ucData[2*i+1]);							// low byte
		if ((m != -1) && (n != -1) && (ucLen < RX_BUF_SIZE))		    // No conversion errors? Receive buffer not full?
			USB_Buffer.ucData[ucLen++] =  m + n;								
	}
	USB_Buffer.ucLength = ucLen;
	
	// Start Byte is present?					  Indicated packet length,   Actual packet length match?
	if ((USB_Buffer.ucData[0] == 0x01) && (USB_Buffer.ucData[1] == USB_Buffer.ucLength-3))	
	{
		// (Value in the Length field does not include the Start, Length, and LRC bytes)
		// Extract the indicated length, which is the packet length starting with the CMD byte.		
		USB_Buffer.ucLengthField = USB_Buffer.ucData[1];		
		
		for (i=1;i<USB_Buffer.ucLengthField+2;i++)
			lrc_calc ^= USB_Buffer.ucData[i];				// Do LRC calculation, starting with the Length field.
		
		if (lrc_calc == USB_Buffer.ucData[i])				// Do LRC check
		{
			if (USB_Buffer.ucData[2] & 0x01)				// LRC good; extract mode bit
				return 1;									// Setup mode
			else																	
				return 0;									// Normal mode											
		}
		else
			return -1;										// bad LRC
	}
	else
		return -2;											// Start or Length byte error
}


void UplinkUSB(void)
{
  unsigned int i;
  //	Packet transmission to the host PC over USB.
  //	Each byte to transmit results in two ASCII characters.
  //	e.g. uncoded 0x01 is sent coded as (0x30, 0x31) == ('0', '1' in ASCII).

  SendByteUSB(0x0D);		// line feed
  SendByteUSB(0x0A);		// carriage return
  SendByteUSB(0x20);		// Space

  for(i=0;i<USB_Buffer.ucLengthActual;i++)
    {
      SendByteUSB(toASCII(USB_Buffer.ucData[i] / 16));		// high ASCII character
      SendByteUSB(toASCII(USB_Buffer.ucData[i] % 16));		// low ASCII character
    }
  SendByteUSB(0x0D);		// CR
  SendByteUSB(0x0A);		// LF
}





